import requests
from bs4 import BeautifulSoup
import dateparser
from espncricinfo.exceptions import PlayerNotFoundError
from espncricinfo.match import Match

class Player(object):

    def __init__(self, player_id):
        self.url = "https://www.espncricinfo.com/ci/content/player/{0}.html".format(str(player_id))
        self.json_url = "http://core.espnuk.org/v2/sports/cricket/athletes/{0}".format(str(player_id))
        self.parsed_html = self.get_html()
        self.json = self.get_json()
        self.player_information = self._parse_player_information()
        self.cricinfo_id = str(player_id)
        if self.parsed_html:
            self.__unicode__ = self._full_name()
            self.name = self._name()
            self.first_name = self._first_name()
            self.full_name = self._full_name()
            self.date_of_birth = self._date_of_birth()
            self.current_age = self._current_age()
            self.major_teams = self._major_teams()
            self.nickname = self._nickname()
            self.playing_role = self._playing_role()
            self.batting_style = self._batting_style()
            self.bowling_style = self._bowling_style()
            self.batting_fielding_averages = self._batting_fielding_averages()
            self.bowling_averages = self._bowling_averages()
            self.test_debut = self._test_debut()
            self.last_test = self._last_test()
            self.t20i_debut = self._t20i_debut()
            self.last_t20i = self._last_t20i()
            self.first_class_debut = self._first_class_debut()
            self.last_first_class = self._last_first_class()
            self.list_a_debut = self._list_a_debut()
            self.last_list_a = self._last_list_a()
            self.t20_debut = self._t20_debut()
            self.last_t20 = self._last_t20()
            self.odi_debut = self._odi_debut()
            self.last_odi = self._last_odi()
            self.recent_matches = self._recent_matches()

    def get_html(self):
        r = requests.get(self.url)
        if r.status_code == 404:
            raise PlayerNotFoundError
        else:
            soup = BeautifulSoup(r.text, 'html.parser')
            return soup.find("div", class_="pnl490M")

    def get_json(self):
        r = requests.get(self.json_url)
        if r.status_code == 404:
            raise PlayerNotFoundError
        else:
            return r.json()

    def _parse_player_information(self):
        return self.parsed_html.find_all('p', class_='ciPlayerinformationtxt')

    def _name(self):
        return self.json['name']

    def _first_name(self):
        return self.json['firstName']

    def _middle_name(self):
        return self.json['middleName']

    def _last_name(self):
        return self.json['lastName']

    def _full_name(self):
        return self.json['fullName']

    def _date_of_birth(self):
        return self.json['dateOfBirth']

    def _current_age(self):
        return self.json['age']

    def _major_teams(self):
        return next((p.text.replace('Major teams ','').split(', ') for p in self.player_information if p.find('b').text == 'Major teams'), None)

    def _nickname(self):
        return next((p.find('span').text for p in self.player_information if p.find('b').text == 'Nickname'), None)

    def _also_known_as(self):
        return next((p.find('span').text for p in self.player_information if p.find('b').text == 'Also known as'), None)

    def _playing_role(self):
        return self.json['position']

    def _batting_style(self):
        return next((x for x in self.json['style'] if x['type'] == 'batting'), None)

    def _bowling_style(self):
        return next((x for x in self.json['style'] if x['type'] == 'bowling'), None)

    def _batting_fielding_averages(self):
        if len(self.parsed_html.findAll('table', class_='engineTable')) == 4:
            headers = ['matches', 'innings', 'not_out', 'runs', 'high_score', 'batting_average', 'balls_faced', 'strike_rate', 'centuries', 'fifties', 'fours', 'sixes', 'catches', 'stumpings']
            bat_field = [td.text.strip() for td in self.parsed_html.find('table', class_='engineTable').findAll('td')]
            num_formats = int(len(bat_field)/15)
            format_positions = [15*x for x in range(num_formats)]
            formats = [bat_field[x] for x in format_positions]
            avg_starts = [x+1 for x in format_positions[:num_formats]]
            avg_finish = [x+14 for x in avg_starts]
            format_averages = [bat_field[x:y] for x,y in zip(avg_starts, avg_finish)]
            combined = list(zip(formats, format_averages))
            l = [{x: dict(zip(headers, y))} for x,y in combined]
            return { k: v for d in l for k, v in d.items() }
        else:
            return None

    def _bowling_averages(self):
        if len(self.parsed_html.findAll('table', class_='engineTable')) == 4:
            headers = ['matches', 'innings', 'balls_delivered', 'runs', 'wickets', 'best_innings', 'best_match', 'bowling_average', 'economy', 'strike_rate', 'four_wickets', 'five_wickets', 'ten_wickets']
            bowling = [td.text.strip() for td in self.parsed_html.findAll('table', class_='engineTable')[1].findAll('td')]
            num_formats = int(len(bowling)/14)
            format_positions = [14*x for x in range(num_formats)]
            formats = [bowling[x] for x in format_positions]
            avg_starts = [x+1 for x in format_positions[:num_formats]]
            avg_finish = [x+13 for x in avg_starts]
            format_averages = [bowling[x:y] for x,y in zip(avg_starts, avg_finish)]
            combined = list(zip(formats, format_averages))
            l = [{x: dict(zip(headers, y))} for x,y in combined]
            return { k: v for d in l for k, v in d.items() }
        else:
            return None

    def _debuts_and_lasts(self):
        if len(self.parsed_html.findAll('table', class_='engineTable')) == 4:
            return self.parsed_html.findAll('table', class_='engineTable')[2]
        else:
            return None

    def _test_debut(self):
        if self._debuts_and_lasts() is not None:
            test_debut = next((tr for tr in self._debuts_and_lasts().findAll('tr') if tr.find('b').text == 'Test debut'), None)
            if test_debut:
                url = 'http://www.espncricinfo.com'+test_debut.find('a')['href']
                match_id = int(test_debut.find('a')['href'].split('/', 4)[4].split('.')[0])
                title = test_debut.findAll('td')[1].text.replace(' scorecard','')
                return {'url': url, 'match_id': match_id, 'title': title}
            else:
                return None
        else:
            return None

    def _last_test(self):
        if self._debuts_and_lasts() is not None:
            last_test = next((tr for tr in self._debuts_and_lasts().findAll('tr') if tr.find('b').text == 'Last Test'), None)
            if last_test:
                url = 'http://www.espncricinfo.com'+last_test.find('a')['href']
                match_id = int(last_test.find('a')['href'].split('/', 4)[4].split('.')[0])
                title = last_test.findAll('td')[1].text.replace(' scorecard','')
                return {'url': url, 'match_id': match_id, 'title': title}
            else:
                return None
        else:
            return None

    def _t20i_debut(self):
        if self._debuts_and_lasts() is not None:
            t20i_debut = next((tr for tr in self._debuts_and_lasts().findAll('tr') if tr.find('b').text == 'T20I debut'), None)
            if t20i_debut:
                url = 'http://www.espncricinfo.com'+t20i_debut.find('a')['href']
                match_id = int(t20i_debut.find('a')['href'].split('/', 4)[4].split('.')[0])
                title = t20i_debut.findAll('td')[1].text.replace(' scorecard','')
                return {'url': url, 'match_id': match_id, 'title': title}
            else:
                return None
        else:
            return None

    def _last_t20i(self):
        if self._debuts_and_lasts() is not None:
            last_t20i = next((tr for tr in self._debuts_and_lasts().findAll('tr') if tr.find('b').text == 'Last T20I'), None)
            if last_t20i:
                url = 'http://www.espncricinfo.com'+last_t20i.find('a')['href']
                match_id = int(last_t20i.find('a')['href'].split('/', 4)[4].split('.')[0])
                title = last_t20i.findAll('td')[1].text.replace(' scorecard','')
                return {'url': url, 'match_id': match_id, 'title': title}
            else:
                return None
        else:
            return None

    def _first_class_debut(self):
        if self._debuts_and_lasts() is not None:
            first_class_debut = next((tr for tr in self._debuts_and_lasts().findAll('tr') if tr.find('b').text == 'First-class debut'), None)
            if first_class_debut:
                try:
                    url = 'http://www.espncricinfo.com'+first_class_debut.find('a')['href']
                    match_id = int(first_class_debut.find('a')['href'].split('/', 4)[4].split('.')[0])
                    title = first_class_debut.findAll('td')[1].text.replace(' scorecard','')
                    return {'url': url, 'match_id': match_id, 'title': title}
                except:
                    return {'url': None, 'match_id': None, 'title': first_class_debut.findAll('td')[1].text}
            else:
                return None
        else:
            return None

    def _last_first_class(self):
        if self._debuts_and_lasts() is not None:
            last_first_class = next((tr for tr in self._debuts_and_lasts().findAll('tr') if tr.find('b').text == 'Last First-class'), None)
            if last_first_class:
                url = 'http://www.espncricinfo.com'+last_first_class.find('a')['href']
                match_id = int(last_first_class.find('a')['href'].split('/', 4)[4].split('.')[0])
                title = last_first_class.findAll('td')[1].text.replace(' scorecard','')
                return {'url': url, 'match_id': match_id, 'title': title}
            else:
                return None
        return None

    def _list_a_debut(self):
        if self._debuts_and_lasts() is not None:
            list_a_debut = next((tr for tr in self._debuts_and_lasts().findAll('tr') if tr.find('b').text == 'List A debut'), None)
            if list_a_debut:
                try:
                    url = 'http://www.espncricinfo.com'+list_a_debut.find('a')['href']
                    match_id = int(list_a_debut.find('a')['href'].split('/', 4)[4].split('.')[0])
                    title = list_a_debut.findAll('td')[1].text.replace(' scorecard','')
                    return {'url': url, 'match_id': match_id, 'title': title}
                except:
                    return {'url': None, 'match_id': None, 'title': list_a_debut.findAll('td')[1].text}
            else:
                return None
        else:
            return None

    def _last_list_a(self):
        if self._debuts_and_lasts() is not None:
            last_list_a = next((tr for tr in self._debuts_and_lasts().findAll('tr') if tr.find('b').text == 'Last List A'), None)
            if last_list_a:
                url = 'http://www.espncricinfo.com'+last_list_a.find('a')['href']
                match_id = int(last_list_a.find('a')['href'].split('/', 4)[4].split('.')[0])
                title = last_list_a.findAll('td')[1].text.replace(' scorecard','')
                return {'url': url, 'match_id': match_id, 'title': title}
            else:
                return None
        else:
            return None

    def _t20_debut(self):
        if self._debuts_and_lasts() is not None:
            t20_debut = next((tr for tr in self._debuts_and_lasts().findAll('tr') if tr.find('b').text == 'Twenty20 debut'), None)
            if t20_debut:
                url = 'http://www.espncricinfo.com'+t20_debut.find('a')['href']
                match_id = int(t20_debut.find('a')['href'].split('/', 4)[4].split('.')[0])
                title = t20_debut.findAll('td')[1].text.replace(' scorecard','')
                return {'url': url, 'match_id': match_id, 'title': title}
            else:
                return None
        else:
            return None

    def _last_t20(self):
        if self._debuts_and_lasts() is not None:
            last_t20 = next((tr for tr in self._debuts_and_lasts().findAll('tr') if tr.find('b').text == 'Last Twenty20'), None)
            if last_t20:
                url = 'http://www.espncricinfo.com'+last_t20.find('a')['href']
                match_id = int(last_t20.find('a')['href'].split('/', 4)[4].split('.')[0])
                title = last_t20.findAll('td')[1].text.replace(' scorecard','')
                return {'url': url, 'match_id': match_id, 'title': title}
            else:
                return None
        else:
            return None

    def _odi_debut(self):
        if self._debuts_and_lasts() is not None:
            odi_debut = next((tr for tr in self._debuts_and_lasts().findAll('tr') if tr.find('b').text == 'ODI debut'), None)
            if odi_debut:
                url = 'http://www.espncricinfo.com'+odi_debut.find('a')['href']
                match_id = int(odi_debut.find('a')['href'].split('/', 4)[4].split('.')[0])
                title = odi_debut.findAll('td')[1].text.replace(' scorecard','')
                return {'url': url, 'match_id': match_id, 'title': title}
            else:
                return None
        else:
            return None

    def _last_odi(self):
        if self._debuts_and_lasts() is not None:
            last_odi = next((tr for tr in self._debuts_and_lasts().findAll('tr') if tr.find('b').text == 'Last ODI'), None)
            if last_odi:
                url = 'http://www.espncricinfo.com'+last_odi.find('a')['href']
                match_id = int(last_odi.find('a')['href'].split('/', 4)[4].split('.')[0])
                title = last_odi.findAll('td')[1].text.replace(' scorecard','')
                return {'url': url, 'match_id': match_id, 'title': title}
            else:
                return None
        else:
            return None

    def _recent_matches(self):
        try:
            table = self.parsed_html.findAll('table', class_='engineTable')[-1]
            return [x.find('a')['href'].split('/', 4)[4].split('.')[0] for x in table.findAll('tr')[1:]]
        except:
            return None

    def in_team_for_match(self, match_id):
        m = Match(match_id)
        if next((p for p in m.team_1_players if p['object_id'] == self.cricinfo_id), None) or next((p for p in m.team_2_players if p['object_id'] == self.cricinfo_id), None):
            return True
        else:
            return False

    def batting_for_match(self, match_id):
        batting_stats = []
        m = Match(match_id)
        for innings in list(m.full_scorecard['innings'].keys()):
            stats = next((x for x in m.full_scorecard['innings'][innings]['batsmen'] if x['href'] == self.url), None)
            if stats:
                batting_stats.append({ 'innings': innings, 'balls_faced': next((x['value'] for x in stats['stats'] if x['name'] == 'ballsFaced'), None), 'minutes': next((x['value'] for x in stats['stats'] if x['name'] == 'minutes'), None), 'runs': next((x['value'] for x in stats['stats'] if x['name'] == 'runs'), None), 'fours': next((x['value'] for x in stats['stats'] if x['name'] == 'fours'), None), 'sixes': next((x['value'] for x in stats['stats'] if x['name'] == 'sixes'), None), 'strike_rate': next((x['value'] for x in stats['stats'] if x['name'] == 'strikeRate'), None) })
        return batting_stats

    def bowling_for_match(self, match_id):
        bowling_stats = []
        m = Match(match_id)
        for innings in list(m.full_scorecard['innings'].keys()):
            stats = next((x for x in m.full_scorecard['innings'][innings]['bowlers'] if x['href'] == self.url), None)
            if stats:
                bowling_stats.append({ 'innings': innings, 'overs': next((x['value'] for x in stats['stats'] if x['name'] == 'overs')), 'maidens': next((x['value'] for x in stats['stats'] if x['name'] == 'maidens')), 'conceded': next((x['value'] for x in stats['stats'] if x['name'] == 'conceded')), 'wickets': next((x['value'] for x in stats['stats'] if x['name'] == 'wickets')), 'economy_rate': next((x['value'] for x in stats['stats'] if x['name'] == 'economyRate')), 'dots': next((x['value'] for x in stats['stats'] if x['name'] == 'dots'), None), 'fours_conceded': next((x['value'] for x in stats['stats'] if x['name'] == 'foursConceded'), None), 'sixes_conceded': next((x['value'] for x in stats['stats'] if x['name'] == 'sixesConceded'), None), 'wides': next((x['value'] for x in stats['stats'] if x['name'] == 'wides'), None), 'no_balls': next((x['value'] for x in stats['stats'] if x['name'] == 'noballs'), None)})
        return bowling_stats



import requests
import grequests
from bs4 import BeautifulSoup
from espncricinfo.exceptions import MatchNotFoundError, NoSeriesError

class Series(object):

    def __init__(self, series_id):
        self.series_id = series_id
        self.json_url = "http://core.espnuk.org/v2/sports/cricket/leagues/{0}/".format(str(series_id))
        self.events_url = "http://core.espnuk.org/v2/sports/cricket/leagues/{0}/events".format(str(series_id))
        self.seasons_url = "http://core.espnuk.org/v2/sports/cricket/leagues/{0}/seasons".format(str(series_id))
        self.json = self.get_json(self.json_url)
        self.seasons = self._get_seasons()
        self.years = self._get_years_from_seasons()
        if self.json:
            self.name = self.json['name']
            self.short_name = self.json['shortName']
            self.abbreviation = self.json['abbreviation']
            self.slug = self.json['slug']
            self.is_tournament = self.json['isTournament']
            self.url = self.json['links'][0]['href']
            self.events_json = self._get_events()

        if self.events_json:
            self.events = self._build_events()

    def get_json(self, url):
        r = requests.get(url)
        if r.status_code == 404:
            raise "Not Found"
        else:
            return r.json()

    def get_events_for_season(self, season):
        responses = []
        season_events = []
        season_events_url = self.seasons_url+"/{0}/events".format(str(season))
        season_events_json = self.get_json(season_events_url)
        if season_events_json:
            rs = (grequests.get(event['$ref']) for event in season_events_json['items'])
            responses = grequests.map(rs)
            for response in responses:
                event_json = response.json()
                venue_json = self.get_json(event_json['venues'][0]['$ref'])
                season_events.append({"url": event_json['$ref'], "match_id": event_json['id'], "class": event_json['competitions'][0]['class']['generalClassCard'], "date": event_json['date'], "description": event_json['shortDescription'], "venue_url": event_json['venues'][0]['$ref'], "venue": venue_json['fullName']})
            return season_events
        else:
            return None

    def __str__(self):
        return self.name

    def __unicode__(self):
        return self.name

    def _get_seasons(self):
        season_json = self.get_json(self.seasons_url)
        if season_json:
            return [x['$ref'] for x in season_json['items']]
        else:
            return None

    def _get_years_from_seasons(self):
        return [x.split('/')[9] for x in self.seasons]

    def _get_events(self):
        events_json = self.get_json(self.events_url)
        if events_json:
            return [x for x in events_json['items']]
        else:
            return None

    def _build_events(self):
        events = []
        for event in self.events_json:
            events.append(self.get_json(event['$ref']))
        return events



import json
import requests
from bs4 import BeautifulSoup
from espncricinfo.exceptions import MatchNotFoundError, NoScorecardError

class Match(object):

    def __init__(self, match_id):
        self.match_id = match_id
        self.match_url = "https://www.espncricinfo.com/matches/engine/match/{0}.html".format(str(match_id))
        self.json_url = "https://www.espncricinfo.com/matches/engine/match/{0}.json".format(str(match_id))
        self.json = self.get_json()
        self.html = self.get_html()
        self.comms_json = self.get_comms_json()
        if self.json:
            self.__unicode__ = self._description()
            self.status = self._status()
            self.match_class = self._match_class()
            self.season = self._season()
            self.description = self._description()
            self.legacy_scorecard_url = self._legacy_scorecard_url()
            self.series = self._series()
            self.series_name = self._series_name()
            self.series_id = self._series_id()
            self.event_url = "http://core.espnuk.org/v2/sports/cricket/leagues/{0}/events/{1}".format(str(self.series_id), str(match_id))
            self.details_url = self._details_url()
            self.officials = self._officials()
            self.current_summary = self._current_summary()
            self.present_datetime_local = self._present_datetime_local()
            self.present_datetime_gmt = self._present_datetime_gmt()
            self.start_datetime_local = self._start_datetime_local()
            self.start_datetime_gmt = self._start_datetime_gmt()
            self.cancelled_match = self._cancelled_match()
            self.rain_rule = self._rain_rule()
            self.date = self._date()
            self.continent = self._continent()
            self.town_area = self._town_area()
            self.town_name = self._town_name()
            self.town_id = self._town_id()
            self.weather_location_code = self._weather_location_code()
            self.match_title = self._match_title()
            self.result = self._result()
            self.ground_id = self._ground_id()
            self.ground_name = self._ground_name()
            self.lighting = self._lighting()
            self.followon = self._followon()
            self.scheduled_overs = self._scheduled_overs()
            self.innings_list = self._innings_list()
            self.innings = self._innings()
            self.latest_batting = self._latest_batting()
            self.latest_bowling = self._latest_bowling()
            self.latest_innings = self._latest_innings()
            self.latest_innings_fow = self._latest_innings_fow()
            self.team_1 = self._team_1()
            self.team_1_id = self._team_1_id()
            self.team_1_abbreviation = self._team_1_abbreviation()
            self.team_1_players = self._team_1_players()
            self.team_1_innings = self._team_1_innings()
            self.team_1_run_rate = self._team_1_run_rate()
            self.team_1_overs_batted = self._team_1_overs_batted()
            self.team_1_batting_result = self._team_1_batting_result()
            self.team_2 = self._team_2()
            self.team_2_id = self._team_2_id()
            self.team_2_abbreviation = self._team_2_abbreviation()
            self.team_2_players = self._team_2_players()
            self.team_2_innings = self._team_2_innings()
            self.team_2_run_rate = self._team_2_run_rate()
            self.team_2_overs_batted = self._team_2_overs_batted()
            self.team_2_batting_result = self._team_2_batting_result()
            self.home_team = self._home_team()
            self.batting_first = self._batting_first()
            self.match_winner = self._match_winner()
            self.toss_winner = self._toss_winner()
            self.toss_decision = self._toss_decision()
            self.toss_decision_name = self._toss_decision_name()
            self.toss_choice_team_id = self._toss_choice_team_id()
            self.toss_winner_team_id = self._toss_winner_team_id()
            self.espn_api_url = self._espn_api_url()
            # from comms_json
            self.rosters = self._rosters()
            self.all_innings = self._all_innings()
            self.close_of_play = self._close_of_play()


    def __str__(self):
        return self.description

    def __repr__(self):
        return (f'{self.__class__.__name__}('f'{self.match_id!r})')

    def get_json(self):
        r = requests.get(self.json_url)
        if r.status_code == 404:
            raise MatchNotFoundError
        elif 'Scorecard not yet available' in r.text:
            raise NoScorecardError
        else:
            return r.json()

    def get_html(self):
        r = requests.get(self.match_url)
        if r.status_code == 404:
            raise MatchNotFoundError
        else:
            return BeautifulSoup(r.text, 'html.parser')

    def match_json(self):
        return self.json['match']

    def innings_comms_url(self, innings=1, page=1):
        return f"https://hsapi.espncricinfo.com/v1/pages/match/comments?lang=en&leagueId={self.series_id}&eventId={self.match_id}&period={innings}&page={page}&filter=full&liveTest=false"

    def get_comms_json(self):
        try:
            text = self.html.find_all('script')[15].string
            return json.loads(text)
        except:
            return None

    def _espn_api_url(self):
        return "https://site.api.espn.com/apis/site/v2/sports/cricket/{0}/summary?event={1}".format(self.series_id, self.match_id)

    def _legacy_scorecard_url(self):
        return "https://static.espncricinfo.com"+self.match_json()['legacy_url']

    def _details_url(self, page=1, number=1000):
        return self.event_url+"/competitions/{0}/details?page_size={1}&page={2}".format(str(self.match_id), str(number), str(page))

    def __str__(self):
        return self.json['description']

    def __unicode__(self):
        return self.json['description']

    def _status(self):
        return self.match_json()['match_status']

    def _match_class(self):
        if self.match_json()['international_class_card'] != "":
            return self.match_json()['international_class_card']
        else:
            return self.match_json()['general_class_card']

    def _season(self):
        return self.match_json()['season']

    def _description(self):
        return self.json['description']

    def _series(self):
        return self.json['series']

    def _series_name(self):
        try:
            return self.json['series'][-1]['series_name']
        except:
            return None

    def _series_id(self):
        return self.json['series'][-1]['core_recreation_id']

    def _officials(self):
        return self.json['official']

    # live matches only
    def _current_summary(self):
        return self.match_json().get('current_summary')

    def _present_datetime_local(self):
        return self.match_json()['present_datetime_local']

    def _present_datetime_gmt(self):
        return self.match_json()['present_datetime_gmt']

    def _start_datetime_local(self):
        return self.match_json()['start_datetime_local']

    def _start_datetime_gmt(self):
        return self.match_json()['start_datetime_gmt']

    def _cancelled_match(self):
        if self.match_json()['cancelled_match'] == 'N':
            return False
        else:
            return True

    def _rain_rule(self):
        if self.match_json().get('rain_rule') == "1":
            return self.match_json()['rain_rule_name']
        else:
            return None

    def _date(self):
        return self.match_json()['start_date_raw']

    def _continent(self):
        return self.match_json().get('continent_name')

    def _town_area(self):
        return self.match_json().get('town_area')

    def _town_name(self):
        return self.match_json().get('town_name')

    def _town_id(self):
        return self.match_json().get('town_id')

    def _weather_location_code(self):
        return self.match_json().get('weather_location_code')

    def _match_title(self):
        return self.match_json()['cms_match_title']

    def _result(self):
        return self.json['live']['status']

    def _ground_id(self):
        return self.match_json()['ground_id']

    def _ground_name(self):
        return self.match_json()['ground_name']

    def _lighting(self):
        return self.match_json()['floodlit_name']

    def _followon(self):
        if self.match_json().get('followon') == '1':
            return True
        else:
            return False

    def _scheduled_overs(self):
        try:
            return int(self.match_json()['scheduled_overs'])
        except:
            return None

    def _innings_list(self):
        try:
            return self.json['centre']['common']['innings_list']
        except:
            return None

    def _innings(self):
        return self.json['innings']

    def _latest_batting(self):
        try:
            return self.json['centre']['common']['batting']
        except:
            return None

    def _latest_bowling(self):
        try:
            return self.json['centre']['common']['bowling']
        except:
            return None

    def _latest_innings(self):
        try:
            return self.json['centre']['common']['innings']
        except:
            return None

    def _latest_innings_fow(self):
        return self.json['centre'].get('fow')

    def _team_1(self):
        return self.json['team'][0]

    def _team_1_id(self):
        return self._team_1()['team_id']

    def _team_1_abbreviation(self):
        return self._team_1()['team_abbreviation']

    def _team_1_players(self):
        return self._team_1().get('player', [])

    def _team_1_innings(self):
        try:
            return [inn for inn in self.json['innings'] if inn['batting_team_id'] == self._team_1_id()][0]
        except:
            return None

    def _team_1_run_rate(self):
        try:
            return float(self._team_1_innings()['run_rate'])
        except:
            return None

    def _team_1_overs_batted(self):
        try:
            return float(self._team_1_innings()['overs'])
        except:
            return None

    def _team_1_batting_result(self):
        try:
            return self._team_1_innings()['event_name']
        except:
            return None

    def _team_2(self):
        return self.json['team'][1]

    def _team_2_id(self):
        return self._team_2()['team_id']

    def _team_2_abbreviation(self):
        return self._team_2()['team_abbreviation']

    def _team_2_players(self):
        return self._team_2().get('player', [])

    def _team_2_innings(self):
        try:
            return [inn for inn in self.json['innings'] if inn['batting_team_id'] == self._team_2_id()][0]
        except:
            return None

    def _team_2_run_rate(self):
        try:
            return float(self._team_2_innings()['run_rate'])
        except:
            return None

    def _team_2_overs_batted(self):
        try:
            return float(self._team_2_innings()['overs'])
        except:
            return None

    def _team_2_batting_result(self):
        try:
            return self._team_2_innings()['event_name']
        except:
            return None

    def _home_team(self):
        if self._team_1_id() == self.match_json()['home_team_id']:
            return self._team_1_abbreviation()
        else:
            return self._team_2_abbreviation()

    def _batting_first(self):
        if self._team_1_id() == self.match_json()['batting_first_team_id']:
            return self._team_1_abbreviation()
        else:
            return self._team_2_abbreviation()

    def _match_winner(self):
        if self._team_1_id() == self.match_json()['winner_team_id']:
            return self._team_1_abbreviation()
        else:
            return self._team_2_abbreviation()

    def _toss_winner(self):
        if self._team_1_id() == self.match_json()['toss_winner_team_id']:
            return self._team_1_id()
        else:
            return self._team_2_id()

    def _toss_decision(self):
        if self.match_json()['toss_decision'] == '' and len(self.innings) > 0:
            if self.innings[0]['batting_team_id'] == self.toss_winner:
                decision = '1'
            else:
                decision = '2'
        else:
            decision = self.match_json()['toss_decision']
        return decision

    def _toss_decision_name(self):
        if self.match_json()['toss_decision_name'] == '' and len(self.innings) > 0:
            if self.innings[0]['batting_team_id'] == self.toss_winner:
                decision_name = 'bat'
            else:
                decision_name = 'bowl'
        else:
            decision_name = self.match_json()['toss_decision_name']
        return decision_name

    def _toss_choice_team_id(self):
        return self.match_json()['toss_choice_team_id']

    def _toss_winner_team_id(self):
        return self.match_json()['toss_winner_team_id']

    # comms_json methods

    def _rosters(self):
        try:
            return self.comms_json['props']['pageProps']['data']['content']['teams']
        except:
            return None

    def _all_innings(self):
        try:
            return self.comms_json['props']['pageProps']['data']['content']['innings']
        except:
            return None

    def _close_of_play(self):
        try:
            return self.comms_json['props']['pageProps']['data']['content']['closePlay']
        except:
            return None

    def batsmen(self, innings):
        try:
            return self.comms_json['props']['pageProps']['data']['content']['innings'][str(innings)]['batsmen']
        except:
            return None

    def bowlers(self, innings):
        try:
            return self.comms_json['props']['pageProps']['data']['content']['innings'][str(innings)]['bowlers']
        except:
            return None

    def did_not_bat(self, innings):
        try:
            return self.comms_json['props']['pageProps']['data']['content']['innings'][str(innings)]['didNotBat']
        except:
            return None

    def extras(self, innings):
        try:
            return self.comms_json['props']['pageProps']['data']['content']['innings'][str(innings)]['extras']
        except:
            return None

    def fows(self, innings):
        try:
            return self.comms_json['props']['pageProps']['data']['content']['innings'][str(innings)]['fallOfWickets']
        except:
            return None

    @staticmethod
    def get_recent_matches(date=None):
        if date:
            url = "https://www.espncricinfo.com/ci/engine/match/index.html?date=%sview=week" % date
        else:
            url = "https://www.espncricinfo.com/ci/engine/match/index.html?view=week"
        r = requests.get(url)
        soup = BeautifulSoup(r.text, 'html.parser')
        return [x['href'].split('/',4)[4].split('.')[0] for x in soup.findAll('a', href=True, text='Scorecard')]



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name="python-espncricinfo",
      version="0.5.8",
      description="ESPNCricInfo API client",
      license="MIT",
      install_requires=["requests", "bs4", "dateparser"],
      author="Derek Willis",
      author_email="dwillis@gmail.com",
      url="http://github.com/dwillis/python-espncricinfo",
      packages = find_packages(),
      keywords= "espncricinfo cricket t20 odi",
      classifiers=['Development Status :: 4 - Beta'],
      zip_safe = True)



#!/usr/bin/env python

"""
This module contains various exceptions raised by python-espncricinfo.
"""


class MatchNotFoundError(TypeError):
    """
    Exception raised if a match_id is not valid or does not exist.
    """
    pass

class PlayerNotFoundError(TypeError):
    pass

class NoScorecardError(TypeError):
    """
    Exception raised if a match has no scorecard
    """
    pass

class NoSeriesError(TypeError):
    """
    Exception raised if a series_id is not valid or does not exist.
    """
    pass



import json
import requests
import datetime
from bs4 import BeautifulSoup
from espncricinfo.match import Match

class Summary(object):

    def __init__(self):
        self.url = "https://www.espncricinfo.com/scores"
        self.html = self.get_html()
        self.match_ids = self._match_ids()
        self.matches = self._build_matches()

    def get_html(self):
        r = requests.get(self.url)
        if r.status_code == 404:
            raise MatchNotFoundError
        else:
            return BeautifulSoup(r.text, 'html.parser')

    def summary_json(self):
        try:
            text = self.html.find_all('script')[15].contents[0]
            return json.loads(text)
        except:
            return None

    def _match_ids(self):
        matches = [x['id'] for x in self.summary_json()['props']['pageProps']['data']['content']['leagueEvents'][0]['matchEvents']]
        return matches

    def _build_matches(self):
        return [Match(m) for m in self.match_ids]
